require 'test/unit'
require_relative '../lib/file_indexer'

DIRS     = [File.expand_path(File.join(__dir__, 'foo/')) + '/',
            File.expand_path(File.join(__dir__, 'bar/')) + '/']
FILES    = [File.join(DIRS[0], 'a.foo'),
            File.join(DIRS[0], 'b.bar'),
            File.join(DIRS[0], 'c.kek'),
            File.join(DIRS[1], 'a.foo'),
            File.join(DIRS[1], 'b.foo')]
EXTS     = %w[foo bar]
DB_FILE  = File.join(__dir__, 'db_file')
TMP_FILE = File.join(DIRS[0], 'd.foo')

class TestIndexer < Test::Unit::TestCase
  def setup
    DIRS.each { |d| FileUtils.mkdir_p(d) }
    FILES.each { |f| FileUtils.touch(f) }
    @indexer = FileIndexer.new(DIRS, EXTS, DB_FILE) do |f|
      File.basename(f).upcase
    end
  end

  def teardown
    @indexer.stop
    FileUtils.rm(DB_FILE) if File.exist?(DB_FILE)
    FileUtils.rm(TMP_FILE) if File.exist?(TMP_FILE)
    DIRS.each { |d| FileUtils.rm_rf(d) }
  end

  def test_glob
    count = @indexer.dirs.map { |d| @indexer.send(:glob, d).length }.reduce(:+)
    assert(4 == count, "Glob counts #{count} files, not 4")
  end

  def test_index_all
    values = @indexer.index_all.values
    assert(
      ['A.FOO', 'B.BAR', 'B.FOO', 'A.FOO'] == values,
      "Values are #{values}"
    )
  end

  def test_index
    assert(['A.FOO'] == @indexer.index([File.join(DIRS[0], 'a.foo')]).values)
  end

  def test_watch
    @indexer.watch
    sleep 0.1
    FileUtils.touch(TMP_FILE)
    sleep 1
    assert(5 == @indexer.files.values.length)
  end

  def test_write_db
    assert(true == File.exist?(DB_FILE))
  end

  def test_read_db
    assert(4 == @indexer.read_db.values.length)
  end
end
